@@ -163,16 +163,23 @@ void FileList::setRoot(const QString& sRoot)
 	QListViewItem* pItem;
 	QString sPath;
 	
-	// Store the new root
-	m_sRoot = sRoot;
-	
 	// Update all items in the list
 	for (pItem = m_pList->firstChild(); pItem != NULL; 
 		pItem = pItem->nextSibling()) {
 		sPath = pItem->text(2);
-		sPath.replace(m_sRoot, "$");
+		
+		// Restore the full path
+		sPath.replace("$", m_sRoot);
+		
+		// Replace the root with a $ sign
+		if (sRoot != "/")
+			sPath.replace(sRoot, "$");
+		
 		pItem->setText(2, sPath);
 	}
+	
+	// Store the new root
+	m_sRoot = sRoot;
 }
 
 /**
