@@ -34,7 +34,6 @@
 #include <kinputdialog.h>
 #include <kxmlguifactory.h>
 #include <kstatusbar.h>
-#include <kaction.h>
 #include <kurldrag.h>
 #include <kkeydialog.h>
 #include "kscope.h"
@@ -64,6 +63,8 @@
 #include "graphwidget.h"
 #include "makedlg.h"
 #include "welcomedlg.h"
+#include "bookmarksdlg.h"
+#include "kscopeactions.h"
 
 /**
  * Class constructor.
@@ -96,7 +97,16 @@ KScope::KScope(QWidget* pParent, const char* szName) :
 	m_pFileViewDock = createDockWidget("File List Window", QPixmap());
 
 	// Connect menu and toolbar items with the object's slots
-	initActions();
+	m_pActions = new KScopeActions(this);
+	m_pActions->init();
+	m_pActions->slotEnableProjectActions(false);
+	
+	// Show a toolbar show/hide menu
+	setStandardToolBarMenuEnabled(true);
+	
+	// Create the initial GUI (no active part)
+	setXMLFile("kscopeui.rc");
+	createShellGUI();
 	
 	// Create all child widgets
 	initMainWindow();
@@ -132,23 +142,6 @@ KScope::KScope(QWidget* pParent, const char* szName) :
 	// Display the symbol dialogue when the user opens a new query page
 	connect(m_pQueryWidget, SIGNAL(newQuery()), 
 		this, SLOT(slotQueryReference()));
-	
-	// Refresh the file list if files were added to or removed from a project
-	connect(m_pProjMgr, SIGNAL(fileListChanged()), this,
-		SLOT(slotProjectFilesChanged()));
-
-	// Add files to the file list when they are added to the project
-	connect(m_pProjMgr, SIGNAL(filesAdded(const QStringList&)), this,
-		SLOT(slotFilesAdded(const QStringList&)));
-
-	// Store the new file tree root of a project
-	connect(m_pFileView, SIGNAL(rootChanged(const QString&)), this,
-		SLOT(slotProjectRootChanged(const QString&)));
-	
-	// Launch a "Find File" query when the "Find File..." button of the
-	// file tree is clicked
-	connect(m_pFileView, SIGNAL(findFile()), this, 
-		SLOT(slotQueryFile()));
 
 	// Rebuild the project database after a certain time period has elapsed
 	// since the last save
@@ -198,168 +191,6 @@ KScope::~KScope()
 		delete m_pMakeDlg;
 }
 
-/**
- * Connects menu bar and toolbar commands with class members. These members
- * handle the actions associated with each command.
- */
-void KScope::initActions()
-{
-	KToggleAction* pLockAction;
-	
-	// File menu
-	KStdAction::openNew(this, SLOT(slotNewFile()), actionCollection());
-	KStdAction::open(this, SLOT(slotOpenFile()), actionCollection());
-	KStdAction::close(this, SLOT(slotCloseEditor()), actionCollection());
-	KStdAction::quit(this, SLOT(slotClose()), actionCollection());
-
-	(void)new KAction(i18n("Go to File List"), KShortcut("Ctrl+Shift+O"),
-		m_pFileList, SLOT(slotSetFocus()), actionCollection(),
-		"file_open_file_from_list");
-	(void)new KAction(i18n("Save Al&l"), 
-		"save_all",	KShortcut("Ctrl+L"), m_pEditTabs, SLOT(slotSaveAll()),
-		actionCollection(), "file_save_all");
-	
-	// Edit menu
-	m_pExtEditAction = new KAction(i18n("Edit in E&xternal Editor"),
-		KShortcut("Ctrl+E"), this,
-		SLOT(slotExtEdit()), actionCollection(), "edit_external_editor");
-	(void)new KAction(i18n("Go To Tag"), KShortcut("Ctrl+Shift+T"), this,
-		SLOT(slotGotoTag()), actionCollection(), "edit_goto_tag");
-	(void)new KAction(i18n("Complete Symbol"), KShortcut("Ctrl+Space"), this,
-		SLOT(slotCompleteSymbol()), actionCollection(), "edit_comp_symbol");
-		
-	// Project menu
-	(void)new KAction(i18n("&New Project..."), 0, this, 
-		SLOT(slotCreateProject()), actionCollection(), "project_new");
-	(void)new KAction(i18n("&Open Project..."), "project_open", 0, this,
-		SLOT(slotOpenProject()), actionCollection(), "project_open");
-	(void)new KAction(i18n("Open &Cscope.out..."), 0, this,
-		SLOT(slotProjectCscopeOut()), actionCollection(),
-		"project_cscope_out");
-	(void)new KAction(i18n("Add/Remove &Files..."), 0, this,
-		SLOT(slotProjectFiles()), actionCollection(),
-		"project_add_rem_files");
-	(void)new KAction(i18n("&Properties..."), 0, this,
-		SLOT(slotProjectProps()), actionCollection(),
-		"project_properties");
-	(void)new KAction(i18n("&Make Project"), KShortcut("Ctrl+M"), this,
-		SLOT(slotProjectMake()), actionCollection(), "project_make");
-	(void)new KAction(i18n("&Remake Project"), KShortcut("Ctrl+Shift+M"), this,
-		SLOT(slotProjectRemake()), actionCollection(), "project_remake");
-	(void)new KAction(i18n("&Close Project"), 0, this,
-		SLOT(slotCloseProject()), actionCollection(), "project_close");
-
-	// Cscope menu
-	(void)new KAction(i18n("&References..."), KShortcut("Ctrl+0"), this,
-		SLOT(slotQueryReference()), actionCollection(), "cscope_references");
-	(void)new KAction(i18n("&Definition..."), 0, KShortcut("Ctrl+1"), this,
-		SLOT(slotQueryDefinition()), actionCollection(), "cscope_definition");
-	(void)new KAction(i18n("&Called Functions..."), 0, KShortcut("Ctrl+2"),
-		this, SLOT(slotQueryCalled()), actionCollection(), "cscope_called");
-	(void)new KAction(i18n("C&alling Functions..."), 0, KShortcut("Ctrl+3"),
-		this, SLOT(slotQueryCalling()),	actionCollection(), "cscope_calling");
-	(void)new KAction(i18n("Find &Text..."), 0, KShortcut("Ctrl+4"), this,
-		SLOT(slotQueryText()), actionCollection(), "cscope_text");
-	(void)new KAction(i18n("Find &EGrep Pattern..."), 0, KShortcut("Ctrl+5"),
-		this, SLOT(slotQueryPattern()), actionCollection(), "cscope_pattern");
-	(void)new KAction(i18n("Find &File..."), 0, KShortcut("Ctrl+7"), this,
-		SLOT(slotQueryFile()), actionCollection(), "cscope_file");
-	(void)new KAction(i18n("&Including Files..."), 0, KShortcut("Ctrl+8"),
-		this, SLOT(slotQueryIncluding()), actionCollection(),
-		"cscope_including");
-	(void)new KAction(i18n("&Quick Definition"), 0, KShortcut("Ctrl+]"), this,
-		SLOT(slotQueryQuickDef()), actionCollection(), "cscope_quick_def");
-	(void)new KAction(i18n("Call &Graph..."), 0, KShortcut("Ctrl+\\"), this,
-		SLOT(slotCallTree()), actionCollection(), "cscope_call_tree");
-	(void)new KAction(i18n("Re&build database"), "reload", 0, this,
-		SLOT(slotRebuildDB()), actionCollection(),
-		"cscope_rebuild");
-
-	// Go menu
-	(void)new KAction(i18n("P&revious Result"), "up", 
-		QKeySequence(ALT + Key_Up), m_pQueryWidget,
-		SLOT(slotPrevResult()), actionCollection(), "go_prev_result");
-	(void)new KAction(i18n("N&ext Result"), "down", 
-		QKeySequence(ALT + Key_Down), m_pQueryWidget,
-		SLOT(slotNextResult()), actionCollection(),
-		"go_next_result");
-	(void)new KAction(i18n("&Previous Position"), "previous", 
-		QKeySequence(ALT + Key_Left), m_pQueryWidget,
-		SLOT(slotHistoryPrev()), actionCollection(), "go_prev_pos");
-	(void)new KAction(i18n("&Next Position"), "next", 
-		QKeySequence(ALT + Key_Right), m_pQueryWidget,
-		SLOT(slotHistoryNext()), actionCollection(),
-		"go_next_pos");
-	(void)new KAction(i18n("Position &History"), 0, KShortcut("Ctrl+h"),
-		this, SLOT(slotHistoryShow()), actionCollection(),
-		"go_history");
-	(void)new KAction(i18n("&Global Bookmarks"), 0, 
-		QKeySequence(CTRL + SHIFT + Key_G), m_pQueryWidget,
-		SLOT(slotBrowseBookmarks()), actionCollection(), "go_bookmarks");
-
-	// View menu
-	m_pToggleFileViewAction = new KToggleAction(i18n("Show/Hide File List"),
-		"view_sidetree", KShortcut("Ctrl+/"), m_pFileViewDock,
-		SLOT(changeHideShowState()), actionCollection(),
-		"view_toggle_filelist_dock");
-	m_pToggleQueryWindowAction = new KToggleAction(
-		i18n("Show/Hide Query Window"), "view_top_bottom",
-		KShortcut("Ctrl+."), m_pQueryDock, SLOT(changeHideShowState()),
-		actionCollection(), "view_toggle_query_dock");
-	m_pToggleTagListAction = new KToggleAction(i18n("Show/Hide Tag List"),
-		"view_detailed", KShortcut("Ctrl+'"), m_pEditTabs,
-		SLOT(slotToggleTagList()), actionCollection(),
-		"view_toggle_tag_list");
-			
-	// Window menu
-	(void)new KAction(i18n("Close &All"), 0, 0, this,
-		SLOT(slotCloseAllWindows()), actionCollection(), "window_close_all");
-	(void)new KAction(i18n("Go &Left"), 0, 
-		QKeySequence(ALT + SHIFT + Key_Left),
-		m_pEditTabs, SLOT(slotGoLeft()), actionCollection(),
-		"window_go_left");
-	(void)new KAction(i18n("Go &Right"), 0,
-		QKeySequence(ALT + SHIFT + Key_Right),
-		m_pEditTabs, SLOT(slotGoRight()), actionCollection(),
-		"window_go_right");
-		
-	// Settings menu
-	KStdAction::preferences(this, SLOT(slotConfigure()), actionCollection());
-	KStdAction::keyBindings(this, SLOT(slotShortcuts()), actionCollection());
-	
-	// Help menu
-	(void)new KAction(i18n("Show &Welcome Message..."), 0, 0, this,
-		SLOT(slotShowWelcome()), actionCollection(), "help_welcome");
-	
-	// Query widget popup menu
-	(void)new KAction(i18n("&New"), "filenew", 0, m_pQueryWidget,
-		SLOT(slotNewQueryPage()), actionCollection(), "query_new");
-	(void)new KAction(i18n("&Refresh"), "reload", 0, m_pQueryWidget,
-		SLOT(slotRefreshCurrent()), actionCollection(), "query_refresh");
-	pLockAction = new KToggleAction(i18n("&Lock/Unlock"), "encrypted", 0,
-		m_pQueryWidget, SLOT(slotLockCurrent()), actionCollection(),
-		"query_toggle_locked");
-	(void)new KAction(i18n("&Close"), "fileclose", 0, m_pQueryWidget,
-		SLOT(slotCloseCurrent()), actionCollection(), "query_close");
-	
-	m_pExtEditAction->setEnabled(Config().useExtEditor());
-	
-	// Show a toolbar show/hide menu
-	setStandardToolBarMenuEnabled(true);
-	
-	// Create the initial GUI (no active part)
-	setXMLFile("kscopeui.rc");
-	createShellGUI();
-
-	// Associate the "Window" menu with the editor tabs widdget
-	m_pEditTabs->setWindowMenu((QPopupMenu*)factory()->container("window",
-		this));
-
-	// Associate the "Query" popup menu with the query widget
-	m_pQueryWidget->setPageMenu((QPopupMenu*)factory()->container(
-		"query_popup", this), pLockAction);
-}
-
 /**
  * Positions child widgets into their docking stations, and performs some
  * other main window initialisation.
@@ -368,7 +199,8 @@ void KScope::initMainWindow()
 {
 	KStatusBar* pStatus;
 	KDockWidget* pMainDock;
-
+	QPopupMenu* pPopup;
+	
 	// Create the status bar
 	pStatus = statusBar();
 	pStatus->insertItem(i18n(" Line: N/A Col: N/A "), 0, 0, true);
@@ -387,7 +219,7 @@ void KScope::initMainWindow()
 
 	// Update the relevant shell action when the dock is hidden through its
 	// close button
-	connect(m_pQueryDock, SIGNAL(headerCloseButtonClicked()), this,
+	connect(m_pQueryDock, SIGNAL(headerCloseButtonClicked()), m_pActions,
 		SLOT(slotQueryDockClosed()));
 		
 	// Create the file view dock
@@ -396,15 +228,21 @@ void KScope::initMainWindow()
 	
 	// Update the relevant shell action when the dock is hidden through its
 	// close button
-	connect(m_pFileViewDock, SIGNAL(headerCloseButtonClicked()), this,
+	connect(m_pFileViewDock, SIGNAL(headerCloseButtonClicked()), m_pActions,
 		SLOT(slotFileViewDockClosed()));
 	
+	// Associate the "Window" menu with the editor tabs widdget
+	pPopup = (QPopupMenu*)factory()->container("window", this);
+	m_pEditTabs->setWindowMenu(pPopup);
+
+	// Associate the "Query" popup menu with the query widget
+	pPopup = (QPopupMenu*)factory()->container("query_popup", this);
+	m_pQueryWidget->setPageMenu(pPopup, m_pActions->getLockAction());
+	
 	// Restore dock configuration
 	Config().loadWorkspace(this);
 	m_bHideQueryOnSelection = m_pQueryDock->isHidden();
-	m_pToggleFileViewAction->setChecked(m_pFileViewDock->isShown());
-	m_pToggleQueryWindowAction->setChecked(m_pQueryDock->isShown());
-	m_pToggleTagListAction->setChecked(Config().getShowTagList());
+	m_pActions->initLayoutActions();
 }
 
 /**
@@ -441,10 +279,10 @@ bool KScope::queryClose()
 void KScope::slotCreateProject()
 {
 	NewProjectDlg dlg(true, this);
-	ProjectManager::Options opt;
+	ProjectBase::Options opt;
 
 	// Prompt the user to close any active projects
-	if (m_pProjMgr->isOpen()) {
+	if (m_pProjMgr->curProject()) {
 		if (KMessageBox::questionYesNo(0, 
 			i18n("The current project needs to be closed before a new one is"
 			" created.\nWould you like to close it now?")) != 
@@ -477,11 +315,18 @@ void KScope::slotCreateProject()
 void KScope::slotOpenProject()
 {
 	OpenProjectDlg dlg;
-
+	QString sPath;
+	
 	if (dlg.exec() == QDialog::Rejected)
 		return;
 
-	openProject(dlg.getPath());
+	sPath = dlg.getPath();
+	
+	// Check if the path refers to a permanent or temporary project
+	if (QFileInfo(sPath).isDir())
+		openProject(sPath);
+	else
+		openCscopeOut(sPath);
 }
 
 /**
@@ -491,14 +336,28 @@ void KScope::slotOpenProject()
  */
 void KScope::slotProjectFiles()
 {
+	ProjectBase* pProj;
+	
 	// A project must be open
-	if (!m_pProjMgr->isOpen())
+	pProj = m_pProjMgr->curProject();
+	if (!pProj)
 		return;
 
+	// Cannot update the file list of a temporary project
+	if (pProj->isTemporary()) {
+		KMessageBox::error(0, i18n("The Add/Remove Files dialogue is not "
+				"available for temporary projects."));
+		return;
+	}
+
 	// Display the files dialog
-	ProjectFilesDlg dlg(m_pProjMgr, this);
-	if (dlg.exec() == QDialog::Accepted)
-		m_pProjMgr->writeList(&dlg);
+	ProjectFilesDlg dlg((Project*)pProj, this);
+	if (dlg.exec() != QDialog::Accepted)
+		return;
+	
+	// Update the project's file list
+	if (pProj->storeFileList(&dlg))
+		slotProjectFilesChanged();
 }
 
 /**
@@ -509,14 +368,16 @@ void KScope::slotProjectFiles()
  */
 void KScope::slotProjectProps()
 {
-	ProjectManager::Options opt;
+	ProjectBase* pProj;
+	ProjectBase::Options opt;
 	
 	// A project must be open
-	if (!m_pProjMgr->isOpen())
+	pProj = m_pProjMgr->curProject();
+	if (!pProj)
 		return;
 
 	// No properties for a temporary project
-	if (m_pProjMgr->isTemporary()) {
+	if (pProj->isTemporary()) {
 		KMessageBox::error(0, i18n("The Project Properties dialogue is not "
 			"available for temporary projects."));
 		return;
@@ -524,16 +385,16 @@ void KScope::slotProjectProps()
 	
 	// Create the properties dialog
 	NewProjectDlg dlg(false, this);
-	m_pProjMgr->getOptions(opt);
-	dlg.setProperties(m_pProjMgr->getName(), m_pProjMgr->getPath(), opt);
-	
+	pProj->getOptions(opt);
+	dlg.setProperties(pProj->getName(), pProj->getPath(), opt);
+		
 	// Display the properties dialog
 	if (dlg.exec() != QDialog::Accepted)
 		return;
 
 	// Set new properties
 	dlg.getOptions(opt);
-	m_pProjMgr->setOptions(opt);
+	pProj->setOptions(opt);
 	
 	// Reset the CscopeFrontend class and the builder object
 	initCscope();
@@ -541,6 +402,9 @@ void KScope::slotProjectProps()
 	// Set auto-completion parameters
 	SymbolCompletion::initAutoCompletion(opt.bACEnabled, opt.nACMinChars,
 		opt.nACDelay, opt.nACMaxEntries);
+	
+	// Set the source root
+	m_pFileView->setRoot(pProj->getSourceRoot());
 }
 
 /**
@@ -685,7 +549,13 @@ void KScope::slotCallTree()
  */
 void KScope::slotRebuildDB()
 {
-	if (!m_pProjMgr->dbExists()) {
+	ProjectBase* pProj;
+	
+	pProj = m_pProjMgr->curProject();
+	if (!pProj)
+		return;
+	
+	if (!pProj->dbExists()) {
 		m_pProgressDlg = new ProgressDlg(i18n("KScope"), i18n("Please wait "
 			"while KScope builds the database"), this);
 		m_pProgressDlg->setAllowCancel(false);
@@ -738,7 +608,7 @@ void KScope::slotProjectFilesChanged()
 	// Refresh the file list
 	m_pFileList->setUpdatesEnabled(false);
 	m_pFileList->clear();
-	m_pProjMgr->fillList(m_pFileList);
+	m_pProjMgr->curProject()->loadFileList(m_pFileList);
 	m_pFileList->setUpdatesEnabled(true);
 	
 	// Rebuild the symbol database
@@ -799,15 +669,16 @@ void KScope::slotQuery(uint nType, bool bPrompt)
 
 /**
  * Opens a project.
- * If another project is currently active, it is cosed first.
+ * If another project is currently active, it is closed first.
  * @param	sDir	The directory of the project to open.
  */
 void KScope::openProject(const QString& sDir)
 {
+	ProjectBase* pProj;
 	QStringList slQueryFiles;
 	QStringList slCallTreeFiles;
 	QStringList slArgs;
-	ProjectManager::Options opt;
+	ProjectBase::Options opt;
 	
 	// Close the current project (may return false if the user clicks on the
 	// "Cancel" button while prompted to save a file)
@@ -819,45 +690,40 @@ void KScope::openProject(const QString& sDir)
 		return;
 	
 	// Change main window title
-	setCaption(m_pProjMgr->getName());
+	pProj = m_pProjMgr->curProject();
+	setCaption(pProj->getName());
 
 	// Set the root of the file tree
-	m_pFileView->setRoot(m_pProjMgr->getRoot());
+	m_pFileView->setRoot(pProj->getSourceRoot());
 	
 	// Initialise Cscope and create a builder object
 	initCscope();
 	
 	// Set auto-completion parameters
-	m_pProjMgr->getOptions(opt);
+	pProj->getOptions(opt);
 	SymbolCompletion::initAutoCompletion(opt.bACEnabled, opt.nACMinChars,
 		opt.nACDelay, opt.nACMaxEntries);
 	
 	// Create an initial query page
 	m_pQueryWidget->addQueryPage();
 	
+	// Enable project-related actions
+	m_pActions->slotEnableProjectActions(true);
+	
 	// If this is a new project (i.e., no source files are yet included), 
 	// display the project files dialogue
-	if (m_pProjMgr->isEmpty()) {
+	if (pProj->isEmpty()) {
 		slotProjectFiles();
 		return;
 	}
 	
 	// Fill the file list with all files in the project. 
 	m_pFileList->setUpdatesEnabled(false);
-	m_pProjMgr->fillList(m_pFileList);
+	pProj->loadFileList(m_pFileList);
 	m_pFileList->setUpdatesEnabled(true);
 	
-	// Reload previously-open files and their bookmarks
-	loadOpenFiles();
-	loadBookmarks();
-	
-	// Load previously stored queries
-	m_pProjMgr->getQueryFiles(slQueryFiles);
-	m_pQueryWidget->loadPages(m_pProjMgr->getPath(), slQueryFiles);
-	
-	// Load previously stored call trees
-	m_pProjMgr->getCallTreeFiles(slCallTreeFiles);
-	m_pCallTreeMgr->loadOpenDialogs(m_pProjMgr->getPath(), slCallTreeFiles);
+	// Restore the last session
+	restoreSession();
 	
 	// Rebuild the cross-reference database
 	if (isAutoRebuildEnabled()) {
@@ -870,6 +736,48 @@ void KScope::openProject(const QString& sDir)
 	}
 }
 
+/**
+ * Opens a temporary project for a Cscope.out file.
+ * @param	sFilePath	The full path of the Cscope.out file
+ * @return	true if successful, false otherwise
+ */
+bool KScope::openCscopeOut(const QString& sFilePath)
+{
+	ProjectBase* pProj;
+	
+	// Close the current project (may return false if the user clicks on the
+	// "Cancel" button while prompted to save a file)
+	if (!slotCloseProject())
+		return false;
+
+	// Open a temporary project for this cscope.out file
+	if (!m_pProjMgr->openCscopeOut(sFilePath))
+		return false;
+	
+	// Change main window title
+	pProj = m_pProjMgr->curProject();
+	setCaption(pProj->getName());
+	
+	// Set the root folder in the file tree
+	m_pFileView->setRoot(pProj->getSourceRoot());
+	
+	// Initialise Cscope and create a builder object
+	initCscope();
+	
+	// Create an initial query page
+	m_pQueryWidget->addQueryPage();
+	
+	// Enable project-related actions
+	m_pActions->slotEnableProjectActions(true);
+	
+	// Fill the file list with all files in the project. 
+	m_pFileList->setUpdatesEnabled(false);
+	pProj->loadFileList(m_pFileList);
+	m_pFileList->setUpdatesEnabled(true);
+	
+	return true;
+}
+
 /**
  * Opens the most recently used project.
  * This method is called when KScope starts, but only if the relevant 
@@ -883,13 +791,15 @@ void KScope::openLastProject()
 	if (slProjects.empty())
 		return;
 		
+	// Get the project's path
 	sPath = *slProjects.begin();
 
-	// Projects already loaded on other instances of KScope should not be
-	// reloaded.
-	if (ProjectManager::isGloballyOpen(sPath))
+	// Check if the path refers to a temporary project
+	if (!QFileInfo(sPath).isDir()) {
+		openCscopeOut(sPath);
 		return;
-		
+	}
+
 	openProject(sPath);
 }
 
@@ -898,23 +808,31 @@ void KScope::openLastProject()
  * In order to reduce the time required by this operation, the GUI of all
  * but the last editor part is not merged with that of the main window.
  */
-void KScope::loadOpenFiles()
+void KScope::restoreSession()
 {
-	FileLocationList list;
+	ProjectBase* pProj;
+	Project::Session sess;
 	FileLocation* pLoc;
 	EditorPage* pPage;
 	
-	// Get the list of open files
-	list.setAutoDelete(true);
-	m_pProjMgr->getOpenFiles(list);
-	if (list.isEmpty())
+	// A session is available for persistent projects only
+	pProj = m_pProjMgr->curProject();
+	if (!pProj || pProj->isTemporary())
 		return;
 	
+	// Make sure all FileLocation objects are deleted
+	sess.fllOpenFiles.setAutoDelete(true);
+	sess.fllBookmarks.setAutoDelete(true);
+	
+	// Load the session
+	((Project*)pProj)->loadSession(sess);
+	
 	// Do not update the GUI when loading the editor parts of the initially
 	// hidden windows
 	m_bUpdateGUI = false;
 	
-	for (pLoc = list.first(); pLoc != NULL; pLoc = list.next()) {
+	for (pLoc = sess.fllOpenFiles.first(); pLoc != NULL;
+		pLoc = sess.fllOpenFiles.next()) {
 		if (QFile::exists(pLoc->m_sPath)) {
 			pPage = addEditor(pLoc->m_sPath);
 			pPage->setCursorPos(pLoc->m_nLine, pLoc->m_nCol);
@@ -925,22 +843,15 @@ void KScope::loadOpenFiles()
 	m_bUpdateGUI = true;
 
 	// Set the active editor (or choose a default one)
-	if (m_pEditTabs->findEditorPage(m_pProjMgr->getLastOpenFile(), true) ==
-		NULL) {
-		m_pEditTabs->findEditorPage(list.last()->m_sPath, true);
-	}
-}
-
-/**
- * Restores bookmarks saved as part of the project's session.
- * This function is called whenever a project is loaded.
- */
-void KScope::loadBookmarks()
-{
-	FileLocationList list;
+	if (m_pEditTabs->findEditorPage(sess.sLastFile, true) == NULL)
+		m_pEditTabs->findEditorPage(sess.fllOpenFiles.last()->m_sPath, true);
 	
-	m_pProjMgr->getBookmarks(list);
-	m_pEditTabs->setBookmarks(list);
+	// Reload bookmarks
+	m_pEditTabs->setBookmarks(sess.fllBookmarks);	
+	
+	// Load previously stored queries and call trees
+	m_pQueryWidget->loadPages(pProj->getPath(), sess.slQueryFiles);
+	m_pCallTreeMgr->loadOpenDialogs(pProj->getPath(), sess.slCallTreeFiles);
 }
 
 /**
@@ -963,35 +874,8 @@ void KScope::toggleQueryWindow(bool bShow)
 	if (m_pQueryDock->isShown() != bShow)
 		m_pQueryDock->changeHideShowState();
 		
-	// Make sure the action's state matches the widget's
-	m_pToggleQueryWindowAction->setChecked(bShow);
-}
-
-/**
- * Opens a temporary project for a Cscope.out file.
- * @param	sFilePath	The full path of the Cscope.out file
- * @return	true if successful, false otherwise
- */
-bool KScope::openCscopeOut(const QString& sFilePath)
-{	
-	// Close the current project (may return false if the user clicks on the
-	// "Cancel" button while prompted to save a file)
-	if (!slotCloseProject())
-		return false;
-
-	// Open a temporary project for this cscope.out file
-	if (!m_pProjMgr->openCscopeOut(sFilePath))
-		return false;
-	
-	// Change main window title
-	setCaption(m_pProjMgr->getName());
-	
-	// Initialise Cscope and create a builder object
-	initCscope();
-	
-	// Create an initial query page
-	m_pQueryWidget->addQueryPage();
-	return true;
+	// Synchronise with the menu command's state
+	m_pActions->slotQueryDockToggled(bShow);
 }
 
 /**
@@ -1060,12 +944,15 @@ void KScope::verifyCscope()
  */
 void KScope::initCscope()
 {
+	ProjectBase* pProj;
+	
 	// Delete the current object, if one exists
 	if (m_pCscopeBuild)
 		delete m_pCscopeBuild;
 
 	// Initialise CscopeFrontend
-	CscopeFrontend::init(m_pProjMgr->getPath(), m_pProjMgr->getArgs());
+	pProj = m_pProjMgr->curProject();
+	CscopeFrontend::init(pProj->getPath(), pProj->getArgs());
 
 	// Create a persistent Cscope process
 	m_pCscopeBuild = new CscopeFrontend();
@@ -1093,50 +980,60 @@ void KScope::initCscope()
  */
 bool KScope::slotCloseProject()
 {
-	QString sLastOpenFile;
-	FileLocationList lstOpenFiles;
-	FileLocationList lstBookmarks;
-	QStringList slQueryFiles;
-	QStringList slCallTreeFiles;
+	ProjectBase* pProj;
+	Project::Session sess;
 	
 	// Do nothing if no project is open
-	if (!m_pProjMgr->isOpen())
+	pProj = m_pProjMgr->curProject();
+	if (!pProj)
 		return true;
 	
+	// Make sure all FileLocation objects are deleted
+	sess.fllOpenFiles.setAutoDelete(true);
+	sess.fllBookmarks.setAutoDelete(true);
+	
 	// Close all open editor pages
 	if (m_pEditTabs->count() > 0) {
-		sLastOpenFile = m_pEditTabs->getCurrentPage()->getFilePath();
-		m_pEditTabs->getOpenFiles(lstOpenFiles);
-		m_pQueryWidget->getBookmarks(lstBookmarks);
+		// Save session information for persistent projects
+		if (!pProj->isTemporary()) {
+			sess.sLastFile = m_pEditTabs->getCurrentPage()->getFilePath();
+			m_pEditTabs->getOpenFiles(sess.fllOpenFiles);
+			m_pEditTabs->getBookmarks(sess.fllBookmarks);
+		}
+		
 		if (!m_pEditTabs->removeAllPages())
 			return false;
 	}
 	
-	// Store locked queries, and close all query pages
-	m_pQueryWidget->savePages(m_pProjMgr->getPath(), slQueryFiles);
-	m_pQueryWidget->slotCloseAll();
-
-	// Store opened call trees
-	m_pCallTreeMgr->saveOpenDialogs(m_pProjMgr->getPath(), slCallTreeFiles);
-	m_pCallTreeMgr->closeAll();
-	
-	// Save current session
-	lstOpenFiles.setAutoDelete(true);
-	m_pProjMgr->setLastOpenFile(sLastOpenFile);
-	m_pProjMgr->setOpenFiles(lstOpenFiles);
-	m_pProjMgr->setBookmarks(lstBookmarks);
-	m_pProjMgr->setQueryFiles(slQueryFiles);
-	m_pProjMgr->setCallTreeFiles(slCallTreeFiles);
+	// Disable project-related actions
+	m_pActions->slotEnableProjectActions(false);
 	
 	// Destroy the make dialogue
 	if (m_pMakeDlg != NULL) {
-		m_pProjMgr->setMakeParams(m_pMakeDlg->getCommand(),
-			m_pMakeDlg->getDir());
+		// Save session information for persistent projects
+		if (!pProj->isTemporary()) {
+			sess.sMakeCmd = m_pMakeDlg->getCommand();
+			sess.sMakeRoot = m_pMakeDlg->getDir();
+		}
 		
 		delete m_pMakeDlg;
 		m_pMakeDlg = NULL;
 	}
 	
+	// Save session information for persistent projects
+	if (!pProj->isTemporary()) {
+		m_pQueryWidget->savePages(pProj->getPath(), sess.slQueryFiles);
+		m_pCallTreeMgr->saveOpenDialogs(pProj->getPath(), sess.slCallTreeFiles);
+	}
+		
+	// Close all query pages and call trees
+	m_pQueryWidget->slotCloseAll();
+	m_pCallTreeMgr->closeAll();
+	
+	// Store session information for persistent projects
+	if (!pProj->isTemporary())
+		((Project*)pProj)->storeSession(sess);
+	
 	// Close the project in the project manager, and terminate the Cscope
 	// process
 	m_pProjMgr->close();
@@ -1147,6 +1044,9 @@ bool KScope::slotCloseProject()
 	// Clear the contents of the file list
 	m_pFileView->clear();
 
+	// Reset queried symbols history
+	SymbolDlg::resetHistory();
+	
 	return true;
 }
 
@@ -1253,7 +1153,7 @@ void KScope::slotProjectMake()
 		m_pMakeDlg = new MakeDlg();
 		
 		// Set make parameters for this project
-		m_pProjMgr->getMakeParams(sCmd, sDir);
+		m_pProjMgr->curProject()->getMakeParams(sCmd, sDir);
 		m_pMakeDlg->setCommand(sCmd);
 		m_pMakeDlg->setDir(sDir);
 		
@@ -1288,6 +1188,29 @@ void KScope::slotProjectRemake()
 	m_pMakeDlg->slotMake();
 }
 
+/**
+ * Handles the "Go->Global Bookmarks" menu command.
+ * Displays a dialogue with the set of all bookmarks currently set in this
+ * project.
+ */
+void KScope::slotShowBookmarks()
+{
+	BookmarksDlg dlg;
+	QString sPath;
+	uint nLine;
+	
+	// Load the bookmark list
+	m_pEditTabs->showBookmarks(dlg.getView());
+	
+	// Show the dialogue
+	if (dlg.exec() != QDialog::Accepted)
+		return;
+	
+	// Go to the selected bookmark
+	dlg.getBookmark(sPath, nLine);
+	slotShowEditor(sPath, nLine);
+}
+
 /**
  * Prompts the user for a symbol to query.
  * Shows a dialog with a line edit widget, where the user can enter a symbol
@@ -1304,7 +1227,6 @@ bool KScope::getSymbol(uint& nType, QString& sSymbol, bool bPrompt)
 {
 	EditorPage* pPage;
 	QString sSuggested;
-	QStringList slSymHistory;
 	
 	// Set the currently selected text, if any
 	if ((pPage = m_pEditTabs->getCurrentPage()) != NULL)
@@ -1316,18 +1238,13 @@ bool KScope::getSymbol(uint& nType, QString& sSymbol, bool bPrompt)
 		return true;
 	}
 	
-	// Get the list of previously requested symbols
-	m_pProjMgr->getSymHistory(slSymHistory);
-	
 	// Show the symbol dialogue
-	sSymbol = SymbolDlg::promptSymbol(this, nType, sSuggested, slSymHistory);
+	sSymbol = SymbolDlg::promptSymbol(this, nType, sSuggested);
 
 	// Cannot accept empty strings
 	if (sSymbol.isEmpty())
 		return false;
 	
-	// Update the symbol history list from the dialog
-	m_pProjMgr->setSymHistory(slSymHistory);
 	return true;
 }
 
@@ -1342,11 +1259,14 @@ EditorPage* KScope::addEditor(const QString& sFilePath)
 {
 	EditorPage* pPage;
 	QString sAbsFilePath;
-		
+	ProjectBase* pProj;
+	
 	// If the file name is given using a relative path, we need to convert
 	// it to an absolute one
-	if (sFilePath[0] != '/') {
-		sAbsFilePath = QDir::cleanDirPath(m_pProjMgr->getRoot() + "/" +
+	// TODO: Project needs a translatePath() method
+	pProj = m_pProjMgr->curProject();
+	if (sFilePath[0] != '/' && pProj) {
+		sAbsFilePath = QDir::cleanDirPath(pProj->getSourceRoot() + "/" +
 			sFilePath);
 	}
 	else {
@@ -1376,6 +1296,7 @@ EditorPage* KScope::createEditorPage()
 	KTextEditor::Document* pDoc;
 	EditorPage* pPage;
 	QPopupMenu* pMenu;
+	ProjectBase* pProj;
 	
 	// Load a new document part
 	pDoc = m_pEditMgr->add();
@@ -1397,25 +1318,17 @@ EditorPage* KScope::createEditorPage()
 		SLOT(slotShowCursorPos(uint, uint)));
 	
 	// Rebuild the database after a file has changed
-	connect(pPage, SIGNAL(fileSaved(const QString&)), this,
-		SLOT(slotSetRebuildTimer(const QString&)));
+	connect(pPage, SIGNAL(fileSaved(const QString&, bool)), this,
+		SLOT(slotFileSaved(const QString&, bool)));
 	
 	// Handle file drops
 	connect(pPage->getView(), SIGNAL(dropEventPass(QDropEvent*)), this,
 		SLOT(slotDropEvent(QDropEvent*)));
-		
-	// Manage bookmarks
-	connect(pPage, 
-		SIGNAL(bookmarkChanged(const QString&, uint, const QString&, bool)),
-		m_pQueryWidget,
-		SLOT(slotBookmarkChanged(const QString&, uint, const QString&, bool)));
-	
-	connect(pPage, SIGNAL(fileClosed(const QString&)), m_pQueryWidget,
-		SLOT(slotRemoveBookmarks(const QString&)));
 
 	// Apply per-project configuration
-	if (m_pProjMgr->getTabWidth() > 0)
-		pPage->setTabWidth(m_pProjMgr->getTabWidth());
+	pProj = m_pProjMgr->curProject();
+	if (pProj && pProj->getTabWidth() > 0)
+		pPage->setTabWidth(pProj->getTabWidth());
 	
 	return pPage;
 }
@@ -1426,7 +1339,10 @@ EditorPage* KScope::createEditorPage()
  */
 inline bool KScope::isAutoRebuildEnabled()
 {
-	return (m_pProjMgr->getAutoRebuildTime() >= 0);
+	ProjectBase* pProj;
+	
+	pProj = m_pProjMgr->curProject();
+	return (pProj && pProj->getAutoRebuildTime() >= 0);
 }
 
 /**
@@ -1465,8 +1381,14 @@ void KScope::slotChangeEditor(EditorPage* pOldPage, EditorPage* pNewPage)
 		m_pEditMgr->setActivePart(pNewPage->getDocument());
 		pFactory->addClient(pNewPage->getView());
 		m_sCurFilePath = pNewPage->getFilePath();
-		setCaption(m_pProjMgr->getName() + " - " + m_sCurFilePath);
+		setCaption(m_pProjMgr->getProjName() + " - " + m_sCurFilePath);
 	}
+	
+	// Enable/disable file-related actions, if necessary
+	if (pOldPage && !pNewPage)
+		m_pActions->slotEnableFileActions(false);
+	else if (!pOldPage && pNewPage)
+		m_pActions->slotEnableFileActions(true);
 }
 
 /**
@@ -1494,7 +1416,8 @@ void KScope::slotShowEditor(const QString& sFilePath, uint nLine)
 	
 	// Make sure the main window is visible
 	raise();
-
+	setWindowState(windowState() & ~WindowMinimized | WindowActive);
+	
 	if (nLine != 0) {
 		// Set the cursor to the requested line
 		pPage->slotGotoLine(nLine);
@@ -1555,11 +1478,14 @@ void KScope::slotNewFile()
  */
 void KScope::slotOpenFile()
 {
+	ProjectBase* pProj;
 	QStringList slFiles;
 	QStringList::Iterator itr;
 	
 	// Prompt the user for the file(s) to open.
-	slFiles = KFileDialog::getOpenFileNames();
+	pProj = m_pProjMgr->curProject();
+	slFiles = KFileDialog::getOpenFileNames(pProj ? pProj->getSourceRoot() : 
+		QString::null);
 	
 	// Open all selected files.
 	for (itr = slFiles.begin(); itr != slFiles.end(); ++itr) {
@@ -1698,7 +1624,7 @@ void KScope::slotApplyPref()
 	m_pEditMgr->applyPrefs();
 
 	// Enable/disable the external editor menu item
-	m_pExtEditAction->setEnabled(Config().useExtEditor());
+	m_pActions->enableExtEditor(Config().useExtEditor());
 }
 
 /**
@@ -1730,7 +1656,7 @@ void KScope::slotShowCursorPos(uint nLine, uint nCol)
 void KScope::slotFileOpened(EditorPage*, const QString& sFilePath)
 {
 	m_sCurFilePath = sFilePath;
-	setCaption(m_pProjMgr->getName() + " - " + m_sCurFilePath);
+	setCaption(m_pProjMgr->getProjName() + " - " + m_sCurFilePath);
 }
 
 /**
@@ -1739,13 +1665,40 @@ void KScope::slotFileOpened(EditorPage*, const QString& sFilePath)
  * object.
  * The time period before rebuilding is determined on a per-project basis.
  * @param	sPath	The full path of the modified file that caused this event
+ * @param	bIsNew	true if this is a new file, false otherwise
  */
-void KScope::slotSetRebuildTimer(const QString& sPath)
+void KScope::slotFileSaved(const QString& sPath, bool bIsNew)
 {
+	ProjectBase* pProj;
 	int nTime;
 	
+	pProj = m_pProjMgr->curProject();
+	if (!pProj)
+		return;
+	
+	// Prompt the user to add this file to the current project
+	if (bIsNew && !pProj->isTemporary()) {
+		if (KMessageBox::questionYesNo(0, 
+			i18n("Whould you like to add this file to the active project?")) == 
+				  KMessageBox::Yes) {
+			
+			// Add the path to the 'cscope.files' file
+			if (!((Project*)pProj)->addFile(sPath)) {
+				KMessageBox::error(0, i18n("Failed to write the file list."));
+				return;
+			}
+			
+			// Add the path to the file list widget
+			m_pFileList->addItem(sPath);
+			
+			// Rebuild immediately
+			slotRebuildDB();
+			return;
+		}
+	}
+	
 	// Get the project's auto-rebuild time
-	nTime = m_pProjMgr->getAutoRebuildTime();
+	nTime = pProj->getAutoRebuildTime();
 	
 	// Do nothing if the time is set to -1
 	if (nTime == -1)
@@ -1766,16 +1719,6 @@ void KScope::slotSetRebuildTimer(const QString& sPath)
 	m_timerRebuild.start(nTime * 1000, true);
 }
 
-/**
- * Sets a new file tree root for the current project.
- * This slot is connected to the rootChanged() signal of the FileView object.
- * @param	sRoot	The full path of the directory to serve as the new root
- */
-void KScope::slotProjectRootChanged(const QString& sRoot)
-{
-	m_pProjMgr->setRoot(sRoot);
-}
-
 /**
  * Handles file drops inside the editors tab widget.
  * Opens all files dropped over the widget.
@@ -1795,26 +1738,4 @@ void KScope::slotDropEvent(QDropEvent* pEvent)
 		addEditor((*itr).path());
 }
 
-/**
- * Ensures the "Show/Hide Query Window" action is unchecked when the dock
- * is closed through its close button.
- * This slot is conncted to the headerCloseButtonClicked() signal of the
- * query window dock widget.
- */
-void KScope::slotQueryDockClosed()
-{
-	m_pToggleQueryWindowAction->setChecked(false);
-}
-
-/**
- * Ensures the "Show/Hide File View" action is unchecked when the dock
- * is closed through its close button.
- * This slot is conncted to the headerCloseButtonClicked() signal of the
- * file view dock widget.
- */
-void KScope::slotFileViewDockClosed()
-{
-	m_pToggleFileViewAction->setChecked(false);
-}
-
 #include "kscope.moc"
