@@ -57,7 +57,7 @@ int main(int argc, char *argv[])
 	// Create the "About" dialogue
 	KAboutData aboutData( "kscope", I18N_NOOP("KScope"),
 		VERSION, description, KAboutData::License_BSD,
-		"(c) 2003-2006, Elad Lahav", 0, "http://kscope.sourceforge.net",
+		"(c) 2003-2007, Elad Lahav", 0, "http://kscope.sourceforge.net",
 		"elad_lahav@users.sf.net");
 	aboutData.addAuthor("Elad Lahav", "Developer",
 		"elad_lahav@users.sf.net");
@@ -75,23 +75,23 @@ int main(int argc, char *argv[])
 
 	// Create the main window
 	KApplication a;
-	KScope* kscope = new KScope();
-	a.setMainWidget(kscope);
+	KScope* pKScope = new KScope();
+	a.setMainWidget(pKScope);
 	
 	// Display the main window
-	kscope->show();  
+	pKScope->show();  
 
 	// Handle command-line arguments
 	if (pArgs->count() > 0) {
-		kscope->parseCmdLine(pArgs);
+		pKScope->parseCmdLine(pArgs);
 	} else if (Config().getLoadLastProj()) {
-		// No arguments given, reload the last project
-		const QStringList slProjects = Config().getRecentProjects();
-		
-		if (!slProjects.empty())
-			kscope->openProject(*slProjects.begin());
+		// No arguments given, load the most recent project
+		pKScope->openLastProject();
 	}
 	
+	// Make sure Cscope is properly installed
+	pKScope->verifyCscope();
+	
 	// Start the event loop
 	return a.exec();
 }
