@@ -53,8 +53,6 @@ PrefFrontend::PrefFrontend(QWidget* pParent, const char* szName) :
 	// Emit the modified() signal when a new path is set
 	connect(m_pCscopeURL, SIGNAL(textChanged(const QString&)), this,
 		SIGNAL(modified()));
-	connect(m_pVerboseCheck, SIGNAL(toggled(bool)), this,
-		SIGNAL(modified()));
 	connect(m_pCtagsURL, SIGNAL(textChanged(const QString&)), this,
 		SIGNAL(modified()));
 	connect(m_pDotURL, SIGNAL(textChanged(const QString&)), this,
@@ -75,7 +73,6 @@ PrefFrontend::~PrefFrontend()
 void PrefFrontend::load()
 {
 	m_pCscopeURL->lineEdit()->setText(Config().getCscopePath());
-	m_pVerboseCheck->setChecked(Config().useVerboseCscope());
 	m_pCtagsURL->lineEdit()->setText(Config().getCtagsPath());
 	m_pDotURL->lineEdit()->setText(Config().getDotPath());
 }
@@ -86,7 +83,6 @@ void PrefFrontend::load()
 void PrefFrontend::apply()
 {
 	Config().setCscopePath(m_pCscopeURL->url());
-	Config().setVerboseCscope(m_pVerboseCheck->isChecked());
 	Config().setCtagsPath(m_pCtagsURL->url());
 	Config().setDotPath(m_pDotURL->url());
 }
@@ -149,6 +145,10 @@ void PrefFrontend::slotAutoConfigTest(uint nType)
 		m_pScriptText->insert(i18n("Cscope support for line mode verbose "
 			"output..."));
 		break;
+		
+	case ConfigFrontend::CscopeSlowPath:
+		m_pScriptText->insert(i18n("Cscope support slow path definitions... "));
+		break;
 			
 	case ConfigFrontend::CtagsPath:
 		m_pScriptText->insert(i18n("Looking for Ctags..."));
@@ -199,9 +199,12 @@ void PrefFrontend::slotAutoConfigResult(uint nType, const QString& sResult)
 	
 	case ConfigFrontend::CscopeVerbose:
 		m_pScriptText->insert(sLine);
-		m_pVerboseCheck->setChecked(sResult == "Yes");
 		break;
 	
+	case ConfigFrontend::CscopeSlowPath:
+		m_pScriptText->insert(sLine);
+		break;
+		
 	case ConfigFrontend::CtagsPath:
 		m_pScriptText->insert(sLine);
 		if (sResult == "ERROR")
