@@ -93,9 +93,10 @@ QueryView::~QueryView()
  * @param	sFile	The file path
  * @param	sLine	The line number in the above file
  * @param	sText	The line's text
+ * @param	pParent	The parent item (ignored)
  */
 void QueryView::addRecord(const QString& sFunc, const QString& sFile,
-	const QString& sLine, const QString& sText, QListViewItem*)
+	const QString& sLine, const QString& sText, QListViewItem* /* pParent */)
 {
 	QListViewItem* pItem;
 	
@@ -316,10 +317,11 @@ void QueryView::slotCopy(QListViewItem* pItem, int nCol)
  */
 void QueryView::slotFilter(int nCol)
 {
-	SearchResultsDlg dlg;
+	SearchResultsDlg dlg(this);
 	QRegExp re;
 	QListViewItem* pItem;
-		
+	bool bNegate;
+	
 	// Prepare the dialogue
 	dlg.setColumn(nCol);
 	
@@ -329,6 +331,7 @@ void QueryView::slotFilter(int nCol)
 		
 	// Get the selected regular expression
 	dlg.getPattern(re);
+	bNegate = dlg.isNegated();
 	
 	// Disable visual updates while search is in progress
 	setUpdatesEnabled(false);
@@ -337,8 +340,10 @@ void QueryView::slotFilter(int nCol)
 	pItem = firstChild();
 	while (pItem != NULL) {
 		// Filter visible items only
-		if (pItem->isVisible() && re.search(pItem->text(nCol)) == -1)
+		if (pItem->isVisible() &&
+			(re.search(pItem->text(nCol)) == -1) != bNegate) {
 			pItem->setVisible(false);
+		}
 		
 		pItem = pItem->nextSibling();
 	}
