@@ -58,9 +58,9 @@ public:
 	QString getSymbol() const;
 	uint getType() const;
 	
-	static QString promptSymbol(QWidget*, uint&, const QString&,
-		QStringList&);
+	static QString promptSymbol(QWidget*, uint&, const QString&);
 	static uint getQueryType(uint);
+	static void resetHistory() { s_slHistory.clear(); }
 	
 private:
 	/** A cscope process used for symbol completion. */
@@ -74,6 +74,8 @@ private:
 	/** Displays query progress information. */
 	CscopeProgress m_progress;
 	
+	static QStringList s_slHistory;
+	
 private slots:
 	void slotHintClicked();
 	void slotHintDataReady(FrontendToken*);
